<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AlteracaoAutomaticaDeParticularidadeController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    
    protected $environment;
    
    public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, $entityManager, $connection, $supUtilities)
    {
        parent::__construct($crudService, $parameterBag);
        $this->crudService       = $crudService;
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager     = $entityManager;
        $this->connection        = $connection;
        $this->supUtilities      = $supUtilities;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        
        $filterCriteria = $request->getFilterCriteria();
		$filterCriteria->addCondition("P_CDOPERADOR", $this->environment->getCdoperador());
		$conditions = $filterCriteria->getConditions();
    	$dataset = $this->crudService->getFilteredDataSet("alteracaoautomaticadeparticularidade", $conditions)->getRows();
		$response->addDataSet(new DataSet("resp", $dataset));
	}

	public function confirmarWidget(Request\Row $request, Response $response) {
	    $dataset = $request->getRow();
	    $arrayPosition = $dataset[0];
	    $arrayGrid = $dataset[1];
	    foreach($arrayGrid as $row){
	        $this->verificaMarca($arrayPosition, $row);
	        $this.verificaFornecedor($arrayPosition, $row);
	        $this.verificaMarcaFornecedor($arrayPosition, $row);
	        
	    }
	    
	}
	
	public function verificaMarca($arrayPosition, $row){
	    
        
            $sql = "SELECT MARCA.CDMARCA, MARCA.NMMARCA FROM MARCA WHERE MARCA.CDMARCA = :P_CDMARCA AND EXISTS (SELECT 1 FROM MARCPROD WHERE MARCPROD.CDMARCA  = MARCA.CDMARCA AND MARCPROD.CDPRODUTO = :P_CDPRODUTO)";
            $params = [
            'P_CDMARCA'   =>  $arrayPosition['CDMARCA'],
            'P_CDPRODUTO' =>  $row['CDPRODUTO']
            ];
            $std = $this->supUtilities->crud($sql, $params);
            
            if(empty($std) === false){
                
                 $queryBrandParticularity = "UPDATE PARTFILI SET CDMARCA    = :P_CDMARCA WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
                $param = [
                'P_CDMARCA'   => $arrayPosition['CDMARCA'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDTIPPART' => $row['CDTIPPART'],
                'P_CDFILIAL'  => $row['CDFILIAL']
                ];
                $this->supUtilities->crud($queryBrandParticularity, $param);
            }
        }
        
    public function verificaFornecedor($arrayPosition, $row){
	    
        
            $sql = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED AND EXISTS (SELECT 1 FROM PRODFORNCOMP WHERE PRODFORNCOMP.CDFORNECED = FORNECEDOR.CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO)";
            $params = [
            'P_CDMARCA'   =>  $arrayPosition['CDFORNECED'],
            'P_CDPRODUTO' =>  $row['CDPRODUTO']
            ];
            $std = $this->supUtilities->crud($sql, $params);
            
            if(empty($std) === false){
                
                 $queryProviderParticularity = "UPDATE PARTFILI SET CDFORNECED = :P_CDFORNECED WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
                $param = [
                'P_CDMARCA'   => $arrayPosition['CDFORNECED'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDTIPPART' => $row['CDTIPPART'],
                'P_CDFILIAL'  => $row['CDFILIAL']
                ];
                $this->supUtilities->crud($queryProviderParticularity, $param);
            }
        }
        
    public function verificaMarcaFornecedor($arrayPosition, $row){
	    
        
            $sql = "SELECT MARCA.CDMARCA, MARCA.NMMARCA FROM MARCA WHERE MARCA.CDMARCA = :P_CDMARCA AND EXISTS (SELECT 1 FROM MARCPROD WHERE MARCPROD.CDMARCA  = MARCA.CDMARCA AND MARCPROD.CDPRODUTO = :P_CDPRODUTO)";
            $params = [
            'P_CDMARCA'   =>  $arrayPosition['CDMARCA'],
            'P_CDPRODUTO' =>  $row['CDPRODUTO']
            ];
            $std = $this->supUtilities->crud($sql, $params);
            
            $selectQuery = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED AND EXISTS (SELECT 1 FROM PRODFORNCOMP WHERE PRODFORNCOMP.CDFORNECED = FORNECEDOR.CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO)";
            $param = [
            'P_CDMARCA'   =>  $arrayPosition['CDFORNECED'],
            'P_CDPRODUTO' =>  $row['CDPRODUTO']
            ];
            $executeSelect = $this->supUtilities->crud($selectQuery, $param);
            
            if(empty($std) === false && empty($executeSelect) === false){
                
                 $queryBrandProviderParticularity = "UPDATE PARTFILI SET CDMARCA    = :P_CDMARCA, CDFORNECED = :P_CDFORNECED WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
                $parametros = [
                'P_CDMARCA'   => $arrayPosition['CDFORNECED'],    
                'P_CDMARCA'   => $arrayPosition['CDMARCA'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDTIPPART' => $row['CDTIPPART'],
                'P_CDFILIAL'  => $row['CDFILIAL']
                ];
                $this->supUtilities->crud($queryBrandProviderParticularity, $parametros);
            }
        }
}