<?php
/**
 * Compras\Logic\NecessidadeCompras
 * @category Php
 * @package  Compras\Logic\NecessidadeCompras
 * @author
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Compras\Logic\NecessidadeCompras;

use \Geral\Entities\Parcmpsup;
use \Geral\Entities\Parasupri;
use Geral\Entities\Trabnececomp;
use Geral\Util\DateUtil;
use \Geral\Util\NovoCodigo;
use \Compras\Logic\NecessidadeCompras\PeriodicidadeNaoCalculada;
use \Compras\Logic\SolicitacaoCompras\SolicitacaoCompras;
use \Planejamento\Logic\PrevisaoConsumo\PrevisaoConsumo;
use \Estoque\Util\Parametros;
use Vendas\Entities\Subgrprodfil;
use Setforms2\Shared\Registry;

use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;

/**
 * Class NecessidadeCompras
 * @category Php
 * @package  Compras\Logic\NecessidadeCompras
 * @author
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class NecessidadeCompras
{

    const NAO = 'N';
    const SIM = 'S';
    const TODOS = 'A';

    /**
     * Cálculo da necessidade de compras
     *
     * @param String $nrorg             Organização
     * @param String $cdfilial          Unidade
     * @param String $tcdfilial         Todas Unidades
     * @param String $cdgrupfili        Grupo de unidade
     * @param String $dtinicial         Data Inicial
     * @param String $dtfinal           Data Final
     * @param String $cdprodini         Produto Inicial
     * @param String $cdprodfin         Produto Final
     * @param String $peacrescimo       Acréscimo
     * @param String $cdoperador        Operador
     * @param String $calcPeriodicidade Cálculo de periodicidade
     * @param String $extra             Extra
     * @param String $dthrcalcnc        Data-hora do cálculo
     * @param String $idqtdezero        Quantidade de zero
     * @param String $subgrupo          Subgrupo
     * @param String $grupprod          Grupo de produto
     *
     * @throws \Exception
     * @return int|string
     */
    public function calculaNecessidadeCompras(
        $nrorg, $cdfilial, $tcdfilial, $cdgrupfili, $dtinicial, $dtfinal,
        $cdprodini, $cdprodfin, $peacrescimo, $cdoperador, $calcPeriodicidade,
        $extra, $dthrcalcnc, $idqtdezero, $subgrupo, $grupprod
    ) {
        if (empty($cdgrupfili) === true) {
            $cdgrupfili = null;
        }

        if (empty($peacrescimo) === true) {
            $peacrescimo = 0;
        }

        try {
            $dtproc = DateUtil::getDataDeString(
                Parametros::getDataProcessamento()
            );
            $dtproc = DateUtil::adicionaIntervalo(
                $dtproc, 30, DateUtil::DIA
            );
            $dtcalculo = $dtproc->format(DateUtil::FORMATO_DATA);

            if ($calcPeriodicidade === self::SIM) {
                $periodNaoCalculada = new PeriodicidadeNaoCalculada();
                $periodNaoCalculada->executaProcedurePeriodicidadeNaoCalculada(
                    $nrorg, $cdfilial, $cdgrupfili, $cdprodini, $cdprodfin
                );
            }
            // Exibe mensagem:
            // Deseja calcular a previsao de consumo no periodo entre as datas de processamento da filial
            // e o dia $dtcalculo.

            //CALCULAR:
            $previsao = $this->calculaPrevisaoFilial($nrorg, $cdfilial);
            if ($previsao === true) {
                $calcPrevCons = new PrevisaoConsumo();
                $calcPrevCons->executaProcedureCalculoPrevConsumo(
                    $nrorg, $cdfilial, $cdgrupfili, $dtcalculo, $peacrescimo,
                    $cdprodini, $cdprodfin, $cdoperador, $dthrcalcnc
                );
            }

            $codigo_nececomp = $this->novoCodigoNeceCompras($nrorg);

            /*
            $this->executaProcedureNecessidadeDeCompras(
                $nrorg, $filial, $grupoFilial, $dtinicial, $dtfinal, $prodinicial,
                $prodfinal,
                $acrescimo, $operador, $dataAtual, $codigo_nececomp, $extra
            );*/

            $this->executeProcedureNecessidadeCompras(
                $cdfilial, $tcdfilial, $cdgrupfili, $dtinicial, $dtfinal, $cdprodini,
                $cdprodfin, $peacrescimo, $cdoperador, $dthrcalcnc, $idqtdezero,
                $codigo_nececomp, $extra, $subgrupo, $grupprod
            );

            return $codigo_nececomp;

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Calcula previsão da filial
     *
     * @param String $nrorg  Organização
     * @param String $filial Filial
     *
     * @return bool
     */
    public function calculaPrevisaoFilial($nrorg, $filial)
    {
        /** @var $dados Parcmpsup */
        $dados = Parcmpsup::getObjectByWhere(
            'NRORG = ? AND CDFILIAL = ?', array($nrorg, $filial)
        );

        if (is_object($dados)) {
            $calc_previsao = $dados->getIdcalprevcon();
            if ($calc_previsao === self::SIM) {
                return true;
            }
        }

        return false;
    }

    /**
     * Novo código nece compras
     *
     * @param String $nrorg Organização
     *
     * @return int|string
     */
    public function novoCodigoNeceCompras($nrorg)
    {
        $codigo = Novocodigo::geraCodigo($nrorg, 'NECECOMP', 1);
        return $codigo;
    }

    /**
     * Verifica alterações nece
     * @param String $nrorg  Organização
     * @param String $filial Unidade
     *
     * @return bool
     */
    public function verificaAlteracoesNece($nrorg, $filial)
    {
        try {
            /** @var $repository \Geral\Entities\Parasupri */
            $repository = Parasupri::getRepository();
            $bloqueio = $repository->consultaPermisaoAltNec($nrorg, $filial);
            return ($bloqueio === self::SIM);
        } catch (\Exception $e) {
            return false;
        }
    }

    /**
     * Gera solicitação
     *
     * @param String $nrorg       Organização
     * @param String $filial      Unidade
     * @param String $dtfinal     Data Final
     * @param String $dtinicial   Data Inicial
     * @param String $filial_proc Unidade de processamento
     * @param String $codigoNece  Código da necessidade
     * @param String $operador    Operador
     * @param String $extra       Extra
     *
     * @return null
     * @throws \Exception
     */
    public function geraSolicitacao(
        $nrorg, $filial, $dtfinal, $dtinicial, $filial_proc, $codigoNece, $operador,
        $extra
    )
    {
        try {
            // Abre relatorio analicitico da previa da necessidade de compras.
            $param
                = "P_DATAFIN=$dtfinal;P_DATAINI=$dtinicial;P_FILIPROC=$filial_proc;P_NRNECECOMP=$codigoNece;P_CDFILIAL=$filial";

            $Qr2 = new \Setforms2\Runtime\Helper\Report\QR2();
            $Qr2->openReport('CMP03800', $param);

            // Gera a solicitacao para pedido.
            $soliCompras = new SolicitacaoCompras();
            $soliCompras->executaProcedureGeraSolicitacao(
                $nrorg, $filial, $codigoNece, $operador, $extra
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida necessidade de compras
     *
     * @param String $grupoFilial Grupo de unidade
     * @param String $filial      Unidade
     * @param String $dtinicial   Data Inicial
     * @param String $dtfinal     Data Final
     * @param String $prodinicial Produto Inicial
     * @param String $prodfinal   Produto Final
     *
     * @return null
     */
    public function validaNecessidadeDeCompras(
        $grupoFilial, $filial, $dtinicial, $dtfinal, $prodinicial, $prodfinal
    )
    {
        //  if ((empty($filial) AND empty($grupoFilial)) OR (!empty($filial) AND !empty($grupoFilial))) {
        // Apenas um dos campos deve ser preenchido: Filial ou Agrupamento.
        //    throw new \Compras\Logic\NecessidadeCompras\Exceptions\ExceptionPreenchimentoCampos();
        // }

        if (empty($dtinicial) OR empty($dtfinal) OR empty($prodinicial)
            OR empty($prodfinal)
        ) {
            // Preencha os campos obrigatórios.
            \Compras\Logic\SolicitacaoCompras\Exception\CamposNaoPreenchidos::mensagemPadrao(
            );
        }

    }

    /**
     * Executa procedure SF_NECESSIDADE_DE_COMPRAS
     *
     * @param String $nrorg       Organização
     * @param String $filial      Unidade
     * @param String $grupoFilial Grupo de unidade
     * @param String $dtinicial   Data de Inicial
     * @param String $dtfinal     Data Final
     * @param String $prodinicial Produto Inicial
     * @param String $prodfinal   Produto Final
     * @param String $acrescimo   Acréscimo
     * @param String $operador    Operador
     * @param String $dataAtual   Data Atual
     * @param String $nrnececomp  Número necessidade de compras
     * @param String $extra       Extra
     *
     * @throws \Exception
     */
    public function executaProcedureNecessidadeDeCompras(
        $nrorg, $filial, $grupoFilial, $dtinicial, $dtfinal,
        $prodinicial, $prodfinal, $acrescimo, $operador,
        $dataAtual, $nrnececomp, $extra
    )
    {
        try {
            $procedure = new Procedure("NECESSIDADE_DE_COMPRAS");
            self::addParametrosCalcNecesCompras(
                $procedure, $nrorg, $filial, $grupoFilial, $dtinicial, $dtfinal,
                $prodinicial,
                $prodfinal, $acrescimo, $operador, $dataAtual, $nrnececomp, $extra
            );
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Adiciona os parametros da procedure NECESSIDADE_DE_COMPRAS.
     *
     * @static
     *
     * @param Procedure $procedure   Setforms2\Runtime\Helper\Procedure $procedure
     * @param String    $nrorg       Organização
     * @param String    $filial      Unidade
     * @param String    $grupoFilial Grupo de unidade
     * @param String    $dtinicial   Data de Inicial
     * @param String    $dtfinal     Data Final
     * @param String    $prodinicial Produto Inicial
     * @param String    $prodfinal   Produto Final
     * @param String    $acrescimo   Acréscimo
     * @param String    $operador    Operador
     * @param String    $dataAtual   Data Atual
     * @param String    $nrnececomp  Número necessidade de compras
     * @param String    $extra       Extra
     */
    protected static function addParametrosCalcNecesCompras(
        Procedure $procedure, $nrorg, $filial, $grupoFilial,
        $dtinicial, $dtfinal, $prodinicial, $prodfinal, $acrescimo,
        $operador, $dataAtual, $nrnececomp, $extra
    )
    {
//        $procedure->addParam(
//            new Param(
//                'P_NRORG',
//                Param::PARAM_INPUT,
//                $nrorg,
//                Param::PARAM_TYPE_INT
//            )
//        );
        $procedure->addParam(
            new Param(
                'P_CDFILIAL',
                Param::PARAM_INPUT,
                $filial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_TCDFILIAL',
                Param::PARAM_INPUT,
                '',
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDGRUPFILI',
                Param::PARAM_INPUT,
                $grupoFilial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_DTINICIAL',
                Param::PARAM_INPUT,
                $dtinicial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_DTFINAL',
                Param::PARAM_INPUT,
                $dtfinal,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDPRODINI',
                Param::PARAM_INPUT,
                $prodinicial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDPRODFIN',
                Param::PARAM_INPUT,
                $prodfinal,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_PEACRESCIMO',
                Param::PARAM_INPUT,
                $acrescimo,
                Param::PARAM_TYPE_INT
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDOPERADOR',
                Param::PARAM_INPUT,
                $operador,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_DTHRCALCNC',
                Param::PARAM_INPUT,
                $dataAtual,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_IDQTDEZERO',
                Param::PARAM_INPUT,
                self::SIM,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_NRNECECOMP',
                Param::PARAM_INPUT,
                $nrnececomp,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_EXTRA',
                Param::PARAM_INPUT,
                $extra,
                Param::PARAM_TYPE_STR
            )
        );
    }

    public static function executeProcedureNecessidadeCompras(
        $cdfilial, $tcdfilial, $cdgrupfili, $dtinicial, $dtfinal, $cdprodini,
        $cdprodfin, $peacrescimo,
        $cdoperador, $dthrcalcnc, $idqtdezero, $nrnececomp, $extra, $subgrupo,
        $grupprod
    ) {
        if (empty($cdgrupfili) === true) {
            $cdgrupfili = null;
        }

        try {
            $procedure = new Procedure("NECESSIDADE_DE_COMPRAS");
            $procedure->addParam(
                new Param(
                    'P_CDFILIAL',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_TCDFILIAL',
                    Param::PARAM_INPUT,
                    $tcdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDGRUPFILI',
                    Param::PARAM_INPUT,
                    $cdgrupfili,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTINICIAL',
                    Param::PARAM_INPUT,
                    $dtinicial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTFINAL',
                    Param::PARAM_INPUT,
                    $dtfinal,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDPRODINI',
                    Param::PARAM_INPUT,
                    $cdprodini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDPRODFIN',
                    Param::PARAM_INPUT,
                    $cdprodfin,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_PEACRESCIMO',
                    Param::PARAM_INPUT,
                    $peacrescimo,
                    Param::PARAM_TYPE_INT
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDOPERADOR',
                    Param::PARAM_INPUT,
                    $cdoperador,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTHRCALCNC',
                    Param::PARAM_INPUT,
                    $dthrcalcnc,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_IDQTDEZERO',
                    Param::PARAM_INPUT,
                    $idqtdezero,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRNECECOMP',
                    Param::PARAM_INPUT,
                    $nrnececomp,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_EXTRA',
                    Param::PARAM_INPUT,
                    $extra,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDSUBGRUPO',
                    Param::PARAM_INPUT,
                    $subgrupo,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDGRUPPROD',
                    Param::PARAM_INPUT,
                    $grupprod,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->call();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o valor do multisearch da filial concatenado
     *
     * @param String $filial Filial
     *
     * @return array
     */
    public static function retornaFilialMultiSearch($filial)
    {
        $cdfilial = '';
        $tcdfilial = '';

        if ($filial == self::TODOS) {
            $tcdfilial = self::TODOS;
            return array($cdfilial, $tcdfilial);
        } elseif ($filial == self::NAO) {
            return array($cdfilial, $tcdfilial);
        } elseif (is_array($filial)) {
            $concatenado = '_';
            foreach ($filial as $f) {
                $concatenado .= $f['cdfilial'] . '_';
            }
            $cdfilial = $concatenado;
            return array($cdfilial, $tcdfilial);
        }
    }

    /**
     * Retorna o valor do multisearch da filial concatenado
     *
     * @param String $subgrupoV Subgrupo
     * @param String $nrorg     Organização
     * @param String $cdfilial  Filial
     *
     * @return array
     */
    public static function retornaSubGrupoMultiSearch(
        $subgrupoV, $nrorg, $cdfilial
    ) {
        $cdsubgrupo = '';
        $cdgrupprod = '';

        if ($subgrupoV == self::TODOS) {
            /**
             * Subgrupo
             * @var \Vendas\Entities\Subgrprodfil $subgrupo
             */
            $subgrupo = Subgrprodfil::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ?', array($nrorg, $cdfilial)
            );
            if ($subgrupo) {
                $cdsubgrupo = '_';
                foreach ($subgrupo as $valor) {
                    /** @var \Vendas\Entities\Subgrprodfil $valor */
                    $cdsubgrupo .= $valor->getCdsubgrprod();
                    $cdsubgrupo .= '_';
                }
                $cdgrupprod = '_';
                foreach ($subgrupo as $valor) {
                    /** @var \Vendas\Entities\Subgrprodfil $valor */
                    $cdgrupprod .= $valor->getCdgrupprod();
                    $cdgrupprod .= '_';
                }
            }
            return array($cdsubgrupo, $cdgrupprod);
        } elseif (is_array($subgrupoV)) {
            $cdsubgrupo = '_';
            foreach ($subgrupoV as $valor) {
                $cdsubgrupo .= $valor['cdsubgrprod'] . '_';
            }
            $cdgrupprod = '_';
            foreach ($subgrupoV as $valor) {
                $cdgrupprod .= $valor['cdgrupprod'] . '_';
            }
            return array($cdsubgrupo, $cdgrupprod);
        }
    }

    /**
     * Marca o campo IDMARCNECCMP como Sim ou Não
     *
     * @param String $nrorg        Organização
     * @param String $cdfilnececmp Filial
     * @param String $nrnececomp   Número de necessidade de compras
     * @param String $cdprodneccmp Produto
     * @param String $dtentrneccmp Data de entrega
     * @param String $id Sim ou não
     *
     * @throws \Exception
     */
    public static function setIdMarcadoNecessCompras(
        $nrorg, $cdfilnececmp, $nrnececomp, $cdprodneccmp, $dtentrneccmp, $id
    ) {
        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            /**
             * Necessidade de compras
             * @var \Geral\Entities\Trabnececomp $trabnececomp
             */
            $trabnececomp = Trabnececomp::getObjectsByWhere(
                'NRORG = ? CDFILNECECMP = ? AND NRNECECOMP = ? AND CDPRODNECCMP = ? AND DTENTRNECCMP = ?',
                array($nrorg, $cdfilnececmp, $nrnececomp, $cdprodneccmp, $dtentrneccmp)
            );

            foreach ($trabnececomp as $item) {
                /**
                 * @var \Geral\Entities\Trabnececomp  $item
                 */
                $item->setIdmarcneccmp($id);
                $conn->flush();
            }

            //$conn->commit();
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }
}